==> plumber-0.3.2.tgz: package/lib/core.js <==
var highland = require('highland');
var flatten = require('flatten');

function parse(chain) {
    var pipeline = assemble(chain, start());
    return {
        run: function() {
            var single = pipeline.take(1);
            single.each(function(resources) {
                resources.resume();
            });
            return single;
        },
        watch: function() {
            // TODO: support?
            console.error("Watch not supported yet");
        }
    };
}

function assemble(chain, executions) {
    return asPipeline(chain).reduce(function(exec, op) {
        return op(exec);
    }, executions);
}



/* == Internals == */

/**
 * Turn an operation or a chain into a flat pipeline (i.e. array of
 * operations)
 */
function asPipeline(chainOrOperation) {
    return flatten([chainOrOperation]);
}

/**
 * Initial step in pipeline.
 */
function start() {
    // FIXME: which one works best for supporting watch?
    // Infinite stream of empty streams
    // var executions = highland(function(push, next) {
    //     push(null, highland([]));
    //     push(null, highland.nil);
    //     // next();
    // });

    // A stream of a single empty stream
    var executions = highland([highland([])]);

    return executions;
}


// TODO: move elsewhere

function operation(mapper) {
    return function(executions) {
        return executions.map(mapper);
    };
}

/** Helper that applies a function to each resource */
operation.map = function(mapper) {
    return operation(function(resources) {
        return resources.map(mapper);
    });
};

/** Helper that applies a function to each resource, running all in
 * parallel (mapper returns a stream) */
operation.parallelFlatMap = function(mapper) {
    return operation(function(resources) {
        return resources.map(mapper).merge();
        // TODO: or parallel to set a maximum concurrent uploads?
    });
};

/** Helper that passes through existing resources and concatenates extra ones */
operation.concat = function(generator) {
    return operation(function(resources) {
        return resources.concat(generator());
    });
};


module.exports = {
    parse: parse,
    assemble: assemble,
    operation: operation
};

==> plumber-0.4.2.tgz: package/lib/core.js <==
var Rx      = require('rx');
var flatten = require('flatten');

function parse(chain) {
    var pipeline = assemble(chain, start());
    return {
        run: function() {
            return pipeline;
        },
        watch: function() {
            // TODO: support?
            console.error("Watch not supported yet");
        }
    };
}

function assemble(chain, executions) {
    return asPipeline(chain).reduce(function(exec, op) {
        return op(exec).share();
    }, executions);
}



/* == Internals == */

/**
 * Turn an operation or a chain into a flat pipeline (i.e. array of
 * operations)
 */
function asPipeline(chainOrOperation) {
    return flatten([chainOrOperation]);
}

/**
 * Initial step in pipeline.
 */
function start() {
    // A stream of a single empty stream
    var initialResources = Rx.Observable.empty();
    var executions = Rx.Observable.return(initialResources);

    return executions;
}


// TODO: move elsewhere

function operation(mapper) {
    return function(executions) {
        return executions.map(mapper);
    };
}

/** Helper that applies a function to each resource */
operation.map = function(mapper) {
    return operation(function(resources) {
        return resources.map(mapper);
    });
};

/** Helper that applies a function to each resource, running all in
 * parallel (mapper returns a stream) */
operation.parallelFlatMap = function(mapper) {
    return operation(function(resources) {
        return resources.map(mapper).mergeAll();
    });
};

/** Helper that applies a function to each resource, running each
 * sequentially (mapper returns a stream) */
operation.serialFlatMap = function(mapper) {
    return operation(function(resources) {
        return resources.map(mapper).concatAll();
    });
};

/** Helper that passes through existing resources and concatenates an
 * extra one */
operation.concat = function(generator) {
    return operation(function(resources) {
        return resources.concat(generator());
    });
};

/** Helper that passes through existing resources and concatenates
 * extra ones */
operation.concatAll = function(generator) {
    return operation(function(resources) {
        return resources.concatAll(generator());
    });
};


module.exports = {
    parse: parse,
    assemble: assemble,
    operation: operation
};
